/*******************************************************************************
modulo: 					TXmain.c 
versione : 					1.1
autore: 					Marcello Pinna
data:  						17/12/2017
descrizione: 				Utilizzo in modo remoto del modulo NFR24L01+ con ADC (collegare un potenziometro ad AN1)
							La lettura adc inviata (fino a 1023) e quella ricevuta vengono mostrate entrambe su due display lcd
picmicro: 					PIC 18F4550
clock: 						20MHz	(oscillatore interno 8 MHz)

ambiente:					MPLABX IDE 2.05
compilatore:				XC8 1.21

Configurazione hardware:	Freedom I, Freedom II (non testato su Freedom III)
							RE0 = CE
							RE2 = CSN
							RB0 = SDI (MISO)
							RB1 = CLK
							RC7 = SDO (MOSI)
							RA5 = SS  (non collegato)
							
							RA1 = Potenziometro 10k

							Impostare i jumper della Freedom II secondo le istruzioni riportate a pagina 380 del manuale "XC8 step by step" 

							Nota: RE1 non si può usare nella freedom I perchè è collegato al cicalino							

CopyRight 2017 				all rights are reserved
*******************************************************************************/

//************************************************************
// INCLUDE
//************************************************************
#include <xc.h>
#include <stdio.h>
#include <stdlib.h>
#include <pic18f4550.h>
#include "PIC18F4550_config.h"
#include <spi.h>
#include "nrf24l01.h"
#include "LCD_44780.h"
#include "LCD_44780.c"
#include "delay.h"

//************************************************************
// DEFINE 
//************************************************************
#define LINE_SPI_SCK   LATBbits.LATB1         
#define LINE_SPI_SDO   LATCbits.LATC7
#define LINE_SPI_SDI   LATBbits.LATB0
#define LINE_SPI_SS	   LATAbits.LATA5	// non usata

#define LCD_DEFAULT
#define NUMERO_CAMPIONI 8

//************************************************************
// PROGRAMMA PRINCIPALE
//************************************************************	
int main(void) 
{
	// Variabile utilizzata per il numero delle letture
	unsigned char i;
	
	// Variabile per salvare la sommatoria dei dati letti
	unsigned int sommatoria = 0; 

	// Variabile per salvare il valore della conversione
	unsigned int lettura = 0;
	
	// Array per i dati da trasmettere
	unsigned char bufferTX[32];
	
	// Imposto PORTA
	LATA = 0x00;
	TRISA = 0b11011111;		// RA5 impostato in uscita per linea SS
	
	// Imposto PORTB
	LATB = 0x00;
	TRISB = 0b11111101;		// RB0 impostato in ingresso per linea SDI (MISO) - RB1 impostato in uscita per linea SCK (CLOCK)
	
	// Imposto PORTC
	LATC = 0x00;
	TRISC = 0b01111111;		// RC7 impostato in uscita per linea (MOSI)
	
	// Imposto PORTD
	LATD = 0x00;
	TRISD = 0x00;			// Tutta in uscita per pilotaggio display lcd
	
	// Imposto PORTE
	LATE = 0x00;
	TRISE = 0x00;			// RE0 impostato in uscita per linea CE - RE2 impostato in uscita per linea CSN 
		
    // Imposto LCD
    LCD_initialize (20);
	
	// Abilito A0-A1 come ingressi analogici
	// VREF sono impostate a massa e VCC
	ADCON1 = 0b00001101;
	
	// Seleziono AN1 come ingresso
	ADCON0 = 0b00000100;
	
	// Imposto i tempi di conversione e giustificazione a destra
	// TAD : FOSC/16
	// TACQ: 16 TAD
	// Giustificazione destra
	ADCON2 = 0b10110101;
	
	// Abilito l'ADC
	ADCON0 |= 0b00000001;
	
	//attesa prima di iniziare la prima conversione
	//basterebbero 2us
	delay_ms (1);
	
	// Imposto oscillatore interno 8 MHz
	OSCCON = 0b01110010;

	// Inizializzo le linee per l'SPI
    LINE_SPI_SCK = 0x00;
    LINE_SPI_SDO = 0x00;
    LINE_SPI_SDI = 0x01;
	
	// Chiudo l'SPI per evitare eventuali errori)
	CloseSPI() ;
  		
	//Inizializzazione del modulo SPI
	//Baudrate Fosc/16 @20MHz -> 1250000 KHz
	OpenSPI(SPI_FOSC_16,MODE_00,SMPEND);
	
	// Inizializzo il modulo NFR24L01
	NRF24L01_Init_TX();
	
	while(1)
		{
			sommatoria = 0;
			
			// Effettuo 8 letture per fare una media
			for (i =0; i<NUMERO_CAMPIONI; i++)
			{
				// Avvio la conversione Analogico/Digitale
				ADCON0bits.GO = 1;
				
				// Attendo la fine della conversione
				while(ADCON0bits.GO)
					;
				// Prelevo il valore della conversione
				lettura = (((unsigned int) ADRESH) << 8) | ADRESL;
				
				// Sommatoria delle letture fatte
				sommatoria = sommatoria + lettura;
			}
			// Calcolo della media
			sommatoria = sommatoria / NUMERO_CAMPIONI;
			
			// Imposto il display lcd
			LCD_clear ();
			LCD_write_message ("ADC = ");
			
			// Visualizzo sul display il dato che trasmetto
			LCD_goto_xy (8,1);
			LCD_write_integer (sommatoria, 4, ZERO_CLEANING_ON);
			
			delay_ms(50);
			
			// L'adc del pic 18F4550 ha una risoluzione pari a 10 bit (0-1023), il che significa che ogni suo valore intero è composto da due 
			// byte, uno basso ed uno alto, memorizzati nei registri ADRESL e ADRESH. Poichè il payload (pacchetto di dati) minimo che posso 
			// inviare è un byte (max 32), per ricomporre questo intero devo disporre di due byte, sia in TX che in RX, ognuno dei quali mi 
			// serve per inviare i due valori separatamente. Poichè a tutti gli effetti ogni byte rappresenta un carattere da 0 a 255, faccio 
			// un casting da Int a Char per ognuno di essi quando li trasmetto. In fase di ricezione farò la cosa contraria.......
			// CASTING da INT a CHAR per i valori low e high contenuti nei registri ADRESL e ADRESH
			bufferTX[0] = (unsigned char) ADRESL;	// byte meno significativo dell'intero
			bufferTX[1] = (unsigned char) ADRESH;	// byte più significativo dell'intero
				
					// Pulisco i buffer TX e RX
					Clear_BUFFER_TX_RX ();
										
					// Ripulisco il registro STATUS
					Clear_STATUS_reg ();
					
					// Scrivo il buffer TX
					unsigned char i;
					NRF24L01_CSN = 0;                                    
					WriteSPI(W_TX_PAYLOAD);                       
					WriteSPI (bufferTX[0]);
					WriteSPI (bufferTX[1]);
					NRF24L01_CSN = 1;  
					NRF24L01_CE = 1;
					delay_ms(1);
					NRF24L01_CE = 0;
					delay_ms(100);
		}
}